/**
 *	Copyright (c) 2008,  Eckhart Arnold
 *
 * Dieses Programm ist freie Software. Sie können es unter den
 * Bedingungen der GNU General Public License, wie von der Free
 * Software Foundation veröffentlicht, weitergeben und/oder
 * modifizieren, entweder gemäß Version 3 der Lizenz oder (nach
 * Ihrer Option) jeder späteren Version.
 *
 * Die Veröffentlichung dieses Programms erfolgt in der Hoffnung,
 * daß es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE
 * GARANTIE, sogar ohne die implizite Garantie der MARKTREIFE oder
 * der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Details finden
 * Sie in der GNU General Public License.
 *
 * Sie sollten ein Exemplar der GNU General Public License zusammen
 * mit diesem Programm erhalten haben. Falls nicht, siehe
 * <http://www.gnu.org/licenses/>.
 */

package evolution.examples;
 
public enum TSPData {
// possible flags: "tbseuP"	
	QUICK_TEST		 (60, 5, 500, 0.10, 0.001, "t"),
	SIMPLE_CLUSTERS  (200, 20, 1000, 0.40, 0.02, ""),
	SIMPLE_RANDOM    (200,  0, 1000, 0.40, 0.02, ""),
	SIMPLEP_CLUSTERS (200, 20, 1000, 0.40, 0.02, "P"),
	SIMPLEP_RANDOM   (200,  0, 1000, 0.40, 0.02, "P"),
	SEU_CLUSTERS     (200, 20, 1000, 0.40, 0.20, "seu"),
	SEU_RANDOM       (200,  0, 1000, 0.40, 0.20, "seu"),
	BEU_CLUSTERS     (200, 20, 1000, 0.40, 0.20, "beu"),
	BEU_RANDOM       (200,  0, 1000, 0.40, 0.20, "beu"),
	SEP_CLUSTERS     (200, 20, 1000, 0.40, 0.20, "seP"),
	SEP_RANDOM       (200,  0, 1000, 0.40, 0.20, "seP"),
	SU_CLUSTERS      (200, 20, 1000, 0.40, 0.20, "su"),
	SU_RANDOM        (200,  0, 1000, 0.40, 0.20, "su"),	
	BEUP_CLUSTERS    (200, 20, 1000, 0.40, 0.20, "beuP"),
	BEUP_RANDOM      (200,  0, 1000, 0.40, 0.20, "beuP"),
	BUP_CLUSTERS     (200, 20, 1000, 0.40, 0.20, "buP"),
	BUP_RANDOM       (200,  0, 1000, 0.40, 0.20, "buP"),
	TSEU_CLUSTERS    (200, 20, 1000, 0.40, 0.20, "tseu"),
	TSEU_RANDOM      (200,  0, 1000, 0.40, 0.20, "tseu"),	
	REFINED_CLUSTERS (200, 20, 1000, 0.40, 0.20, "tseP"),
	REFINED_RANDOM   (200,  0, 1000, 0.40, 0.20, "tseP"),
	FEW_MUTATIONS_C  (200, 20, 1000, 0.20, 0.02, "tseuP"),
	FEW_MUTATIONS_R  (200,  0, 1000, 0.20, 0.02, "tseuP");

	
	public final int  points, clusters, popSize;
	public final double pCrossover, pMutation;
	public final double[][] location;
	public final String flags;
	public final boolean trimming, boltzmann, steepening, elite, 
	                     universal, pointgenome; 	
	
	TSPData(int points, int clusters, int popSize, 
			double pCrossover, double pMutation, String flags) {
		
		EAUtil.rnd.setSeed(points*100000+clusters*10000+popSize); // slight manipulation to assure equality of data
		
		int clusterSize = clusters == 0 ? points : points/clusters;
		assert clusters == 0 || clusterSize * clusters == points;
		this.points = points;
		this.clusters = clusters;
		this.popSize = popSize;
		this.pCrossover = pCrossover;
		this.pMutation = pMutation / points;
		this.flags = flags;
		if (flags.contains("t")) trimming = true; else trimming = false;
		if (flags.contains("b")) boltzmann = true; else boltzmann = false; 
		if (flags.contains("s")) steepening = true; else steepening = false;
		if (flags.contains("e")) elite = true; else elite = false;
		if (flags.contains("u")) universal = true; else universal = false;	
		if (flags.contains("P")) pointgenome = true; else pointgenome = false;
		double[][] loc = new double[points][2];			
		if (clusters == 0) {
			for (int i=0; i < points; i++) {
				loc[i][0] = EAUtil.rnd.nextDouble();
				loc[i][1] = EAUtil.rnd.nextDouble();
			}
		} else {
			double[][] center = new double[clusters][2];
			double x, y, dx, dy, d;
			double minX=0, minY=0, maxX=0, maxY=0;
			int l;
			if (clusters == 1) {
				center[0][0] = 0.0;  
				center[0][1] = 0.0;
				d = 1.0;
			} else {
				dx = 1.0-Math.cos(2*Math.PI/clusters);
				dy = Math.sin(2*Math.PI/clusters);
				d = Math.sqrt(dx*dx + dy*dy) / 8;
				for (int i = 0; i < clusters; i++) {
					center[i][0] = Math.cos(i*2*Math.PI/clusters);
					center[i][1] = Math.sin(i*2*Math.PI/clusters);
				}					
			}
			for (int i = 0; i < clusters; i++) {
				for (int k = 0; k < clusterSize; k++) {
					x = EAUtil.rnd.nextGaussian()*d + center[i][0];
					y = EAUtil.rnd.nextGaussian()*d + center[i][1];
					l = i*clusterSize+k;
					loc[l][0] = x;
					loc[l][1] = y;
					if (x < minX) minX = x;  if (x > maxX) maxX = x;
					if (y < minY) minY = y;  if (y > maxY) maxY = y;
				}
			}
			if (minX > -0.5) minX = -0.5;  if (maxX < 0.5) maxX = 0.5;
			if (minY > -0.5) minY = -0.5;  if (maxY < 0.5) maxY = 0.5;
			dx = (maxX-minX);
			dy = (maxY-minY);
			for (int i = 0; i < points; i++) {
				loc[i][0] = (loc[i][0] - minX) / dx;
				loc[i][1] = (loc[i][1] - minY) / dy;
			}
			// shuffle the points a bit...
			for (int i = 0; i < points; i++) {
				int k = EAUtil.rnd.nextInt(points);
				double swap[] = loc[i];
				loc[i] = loc[k];
				loc[k] = swap;
			}
		}
		location = loc;
	}
}